"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toComponent = toComponent;

var _utils = require("./utils");

var _renderer = require("./renderer");

function toComponent(decorators) {
  decorators = decorators || [];

  function createComponent(Component, bindings) {
    bindings = bindings || {};
    const mapScopeToProps = (0, _utils.createMapScopeToProps)(bindings);

    class ComponentController {
      constructor($element, $injector) {
        this.$injector = $injector;
        this.WrappedComponent = (0, _utils.wrapWithDecorators)(this.resolveDecorators(decorators))(Component);
        this.renderer = (0, _renderer.createRenderer)(this.WrappedComponent, $element[0]);
      }

      resolveDecorators(decorators) {
        var _this = this;

        return decorators.map(function (decorator) {
          return _this.$injector.invoke(decorator);
        });
      }

      $onChanges() {
        const props = mapScopeToProps(this);
        this.renderer.render(props);
      }

      $onDestroy() {
        this.renderer.unmount();
      }

    }

    ComponentController.$inject = ['$element', '$injector'];
    return {
      bindings: bindings,
      controller: ComponentController
    };
  }

  ; // Exposed for testing

  createComponent._decorators = decorators;
  return createComponent;
}